@@ -23,29 +23,20 @@ Syntax example:
 one line.
 """
 
+from collections import OrderedDict
 from os import path as op
-
+import codecs
 import re
 import sys
 import textwrap
 
+import glob2
+import six
+
 from . import types
 from . import exceptions
 
 
-class FeatureError(Exception):
-
-    """Feature parse error."""
-
-    message = u"{0}.\nLine number: {1}.\nLine: {2}.\nFile: {3}"
-
-    def __str__(self):
-        return unicode(self).encode("utf-8")
-
-    def __unicode__(self):
-        return self.message.format(*self.args)
-
-
 # Global features dictionary
 features = {}
 
@@ -64,9 +55,8 @@ STEP_PREFIXES = [
     ("And ", None),  # Unknown step type,
 ]
 
-COMMENT_SYMBOLS = "#"
-
 STEP_PARAM_RE = re.compile("\<(.+?)\>")
+COMMENT_RE = re.compile('(^|(?<=\s))#')
 
 
 def get_step_type(line):
@@ -88,10 +78,9 @@ def strip_comments(line):
 
     :return: Stripped line.
     """
-    try:
-        line = line[:line.index(COMMENT_SYMBOLS)]
-    except ValueError:
-        pass
+    res = COMMENT_RE.search(line)
+    if res:
+        line = line[:res.start()]
     return line.strip()
 
 
@@ -108,20 +97,6 @@ def parse_line(line):
     return "", line
 
 
-def _open_file(filename, encoding):
-    """Open file for reading (Python compatibility function).
-
-    :param str filename: File name.
-    :param str encoding: Encoding.
-
-    :return: File object.
-    """
-    if sys.version_info < (3, 0):
-        return open(filename, "r")
-    else:
-        return open(filename, "r", encoding=encoding)
-
-
 def force_unicode(obj, encoding="utf-8"):
     """Get the unicode string out of given object (python 2 and python 3).
 
@@ -134,7 +109,7 @@ def force_unicode(obj, encoding="utf-8"):
             return obj.decode(encoding)
         else:
             return unicode(obj)
-    else:
+    else:  # pragma: no cover
         return str(obj)
 
 
@@ -159,29 +134,137 @@ def get_tags(line):
 
     :return: List of tags.
     """
+    if not line or not line.strip().startswith('@'):
+        return set()
     return (
-        set((tag for tag in line.split() if tag.startswith("@") and len(tag) > 1))
-        if line else set()
+        set((tag.lstrip('@') for tag in line.strip().split(' @') if len(tag) > 1))
     )
 
 
+def get_features(paths, **kwargs):
+    """Get features for given paths.
+
+    :param list paths: `list` of paths (file or dirs)
+
+    :return: `list` of `Feature` objects.
+    """
+    seen_names = set()
+    features = []
+    for path in paths:
+        if path not in seen_names:
+            seen_names.add(path)
+            if op.isdir(path):
+                features.extend(
+                    get_features(
+                        glob2.iglob(op.join(path, "**", "*.feature")),
+                        **kwargs
+                    )
+                )
+            else:
+                base, name = op.split(path)
+                feature = Feature.get_feature(base, name, **kwargs)
+                features.append(feature)
+    features.sort(key=lambda feature: feature.name or feature.filename)
+    return features
+
+
+class Examples(object):
+
+    """Example table."""
+
+    def __init__(self):
+        """Initialize examples instance."""
+        self.example_params = []
+        self.examples = []
+        self.vertical_examples = []
+        self.line_number = None
+        self.name = None
+
+    def set_param_names(self, keys):
+        """Set parameter names.
+
+        :param names: `list` of `string` parameter names.
+        """
+        self.example_params = [str(key) for key in keys]
+
+    def add_example(self, values):
+        """Add example.
+
+        :param values: `list` of `string` parameter values.
+        """
+        self.examples.append(values)
+
+    def add_example_row(self, param, values):
+        """Add example row.
+
+        :param param: `str` parameter name
+        :param values: `list` of `string` parameter values
+        """
+        if param in self.example_params:
+            raise exceptions.ExamplesNotValidError(
+                """Example rows should contain unique parameters. "{0}" appeared more than once""".format(
+                    param,
+                )
+            )
+        self.example_params.append(param)
+        self.vertical_examples.append(values)
+
+    def get_params(self, converters, builtin=False):
+        """Get scenario pytest parametrization table.
+
+        :param converters: `dict` of converter functions to convert parameter values
+        """
+        param_count = len(self.example_params)
+        if self.vertical_examples and not self.examples:
+            for value_index in range(len(self.vertical_examples[0])):
+                example = []
+                for param_index in range(param_count):
+                    example.append(self.vertical_examples[param_index][value_index])
+                self.examples.append(example)
+
+        if self.examples:
+            params = []
+            for example in self.examples:
+                example = list(example)
+                for index, param in enumerate(self.example_params):
+                    raw_value = example[index]
+                    if converters and param in converters:
+                        value = converters[param](raw_value)
+                        if not builtin or value.__class__.__module__ in {'__builtin__', 'builtins'}:
+                            example[index] = value
+                params.append(example)
+            return [self.example_params, params]
+        else:
+            return []
+
+    def __bool__(self):
+        """Bool comparison."""
+        return bool(self.vertical_examples or self.examples)
+
+    if six.PY2:
+        __nonzero__ = __bool__
+
+
 class Feature(object):
 
     """Feature."""
 
-    def __init__(self, basedir, filename, encoding="utf-8"):
+    def __init__(self, basedir, filename, encoding="utf-8", strict_gherkin=True):
         """Parse the feature file.
 
         :param str basedir: Feature files base directory.
         :param str filename: Relative path to the feature file.
         :param str encoding: Feature file encoding (utf-8 by default).
+        :param bool strict_gherkin: Flag whether it's a strictly gherkin scenario or not (e.g. it will validate correct
+            gherkin language (given-when-then))
         """
-        self.scenarios = {}
+        self.scenarios = OrderedDict()
         self.rel_filename = op.join(op.basename(basedir), filename)
         self.filename = filename = op.abspath(op.join(basedir, filename))
         self.line_number = 1
         self.name = None
         self.tags = set()
+        self.examples = Examples()
         scenario = None
         mode = None
         prev_mode = None
@@ -191,7 +274,7 @@ class Feature(object):
         prev_line = None
         self.background = None
 
-        with _open_file(filename, encoding) as f:
+        with codecs.open(filename, encoding=encoding) as f:
             content = force_unicode(f.read(), encoding)
             for line_number, line in enumerate(content.splitlines(), start=1):
                 unindented_line = line.lstrip()
@@ -206,31 +289,49 @@ class Feature(object):
                     multiline_step = False
                 stripped_line = line.strip()
                 clean_line = strip_comments(line)
-                if not clean_line:
+                if not clean_line and (not prev_mode or prev_mode not in types.FEATURE):
                     continue
                 mode = get_step_type(clean_line) or mode
 
-                if mode == types.GIVEN and prev_mode not in (
-                        types.GIVEN, types.SCENARIO, types.SCENARIO_OUTLINE, types.BACKGROUND):
-                    raise FeatureError("Given steps must be the first in withing the Scenario",
-                                       line_number, clean_line, filename)
+                allowed_prev_mode = (types.BACKGROUND, types.GIVEN)
+                if not strict_gherkin:
+                    allowed_prev_mode += (types.WHEN, )
 
-                if mode == types.WHEN and prev_mode not in (
-                        types.SCENARIO, types.SCENARIO_OUTLINE, types.GIVEN, types.WHEN):
-                    raise FeatureError("When steps must be the first or follow Given steps",
-                                       line_number, clean_line, filename)
+                if not scenario and prev_mode not in allowed_prev_mode  and mode in types.STEP_TYPES:
+                    raise exceptions.FeatureError(
+                        "Step definition outside of a Scenario or a Background", line_number, clean_line, filename)
 
-                if not self.background and mode == types.THEN and prev_mode not in types.STEP_TYPES:
-                    raise FeatureError("Then steps must follow Given or When steps",
-                                       line_number, clean_line, filename)
+                if strict_gherkin:
+                    if (self.background and not scenario and mode not in (
+                            types.SCENARIO, types.SCENARIO_OUTLINE, types.GIVEN, types.TAG)):
+                        raise exceptions.FeatureError(
+                            "Background section can only contain Given steps", line_number, clean_line, filename)
+
+                    if mode == types.GIVEN and prev_mode not in (
+                            types.GIVEN, types.SCENARIO, types.SCENARIO_OUTLINE, types.BACKGROUND):
+                        raise exceptions.FeatureError(
+                            "Given steps must be the first within the Scenario", line_number, clean_line, filename)
+
+                    if mode == types.WHEN and prev_mode not in (
+                            types.SCENARIO, types.SCENARIO_OUTLINE, types.GIVEN, types.WHEN):
+                        raise exceptions.FeatureError(
+                            "When steps must be the first or follow Given steps", line_number, clean_line, filename)
+
+                    if not self.background and mode == types.THEN and prev_mode not in types.STEP_TYPES:
+                        raise exceptions.FeatureError(
+                            "Then steps must follow Given or When steps", line_number, clean_line, filename)
 
                 if mode == types.FEATURE:
-                    if prev_mode != types.FEATURE:
+                    if prev_mode is None or prev_mode == types.TAG:
                         _, self.name = parse_line(clean_line)
                         self.line_number = line_number
                         self.tags = get_tags(prev_line)
-                    else:
+                    elif prev_mode == types.FEATURE:
                         description.append(clean_line)
+                    else:
+                        raise exceptions.FeatureError(
+                            "Multiple features are not allowed in a single feature file",
+                            line_number, clean_line, filename)
 
                 prev_mode = mode
 
@@ -246,16 +347,29 @@ class Feature(object):
                     )
                 elif mode == types.EXAMPLES:
                     mode = types.EXAMPLES_HEADERS
+                    (scenario or self).examples.line_number = line_number
                 elif mode == types.EXAMPLES_VERTICAL:
                     mode = types.EXAMPLE_LINE_VERTICAL
+                    (scenario or self).examples.line_number = line_number
                 elif mode == types.EXAMPLES_HEADERS:
-                    scenario.set_param_names([l.strip() for l in parsed_line.split("|")[1:-1] if l.strip()])
+                    (scenario or self).examples.set_param_names(
+                        [l.strip() for l in parsed_line.split("|")[1:-1] if l.strip()])
                     mode = types.EXAMPLE_LINE
                 elif mode == types.EXAMPLE_LINE:
-                    scenario.add_example([l.strip() for l in stripped_line.split("|")[1:-1]])
+                    (scenario or self).examples.add_example([l.strip() for l in stripped_line.split("|")[1:-1]])
                 elif mode == types.EXAMPLE_LINE_VERTICAL:
                     param_line_parts = [l.strip() for l in stripped_line.split("|")[1:-1]]
-                    scenario.add_example_row(param_line_parts[0], param_line_parts[1:])
+                    try:
+                        (scenario or self).examples.add_example_row(param_line_parts[0], param_line_parts[1:])
+                    except exceptions.ExamplesNotValidError as exc:
+                        if scenario:
+                            raise exceptions.FeatureError(
+                                """Scenario has not valid examples. {0}""".format(
+                                    exc.args[0]), line_number, clean_line, filename)
+                        else:
+                            raise exceptions.FeatureError(
+                                """Feature has not valid examples. {0}""".format(
+                                    exc.args[0]), line_number, clean_line, filename)
                 elif mode and mode not in (types.FEATURE, types.TAG):
                     step = Step(
                         name=parsed_line,
@@ -264,21 +378,24 @@ class Feature(object):
                         line_number=line_number,
                         keyword=keyword,
                     )
-                    if self.background and mode == types.GIVEN and not scenario:
+                    if self.background and (mode == types.GIVEN or not strict_gherkin) and not scenario:
                         target = self.background
                     else:
                         target = scenario
                     target.add_step(step)
                 prev_line = clean_line
 
-        self.description = u"\n".join(description)
+        self.description = u"\n".join(description).strip()
 
     @classmethod
-    def get_feature(cls, base_path, filename, encoding="utf-8"):
+    def get_feature(cls, base_path, filename, encoding="utf-8", strict_gherkin=True):
         """Get a feature by the filename.
 
         :param str base_path: Base feature directory.
         :param str filename: Filename of the feature file.
+        :param str encoding: Feature file encoding.
+        :param bool strict_gherkin: Flag whether it's a strictly gherkin scenario or not (e.g. it will validate correct
+            gherkin language (given-when-then))
 
         :return: `Feature` instance from the parsed feature cache.
 
@@ -289,7 +406,7 @@ class Feature(object):
         full_name = op.abspath(op.join(base_path, filename))
         feature = features.get(full_name)
         if not feature:
-            feature = Feature(base_path, filename, encoding=encoding)
+            feature = Feature(base_path, filename, encoding=encoding, strict_gherkin=strict_gherkin)
             features[full_name] = feature
         return feature
 
@@ -310,9 +427,7 @@ class Scenario(object):
         self.feature = feature
         self.name = name
         self._steps = []
-        self.example_params = []
-        self.examples = []
-        self.vertical_examples = []
+        self.examples = Examples()
         self.line_number = line_number
         self.example_converters = example_converters
         self.tags = tags or set()
@@ -348,56 +463,14 @@ class Scenario(object):
         """
         return frozenset(sum((list(step.params) for step in self.steps), []))
 
-    def set_param_names(self, keys):
-        """Set parameter names.
+    def get_example_params(self):
+        """Get example parameter names."""
+        return set(self.examples.example_params + self.feature.examples.example_params)
 
-        :param names: `list` of `string` parameter names.
-        """
-        self.example_params = [str(key) for key in keys]
-
-    def add_example(self, values):
-        """Add example.
-
-        :param values: `list` of `string` parameter values.
-        """
-        self.examples.append(values)
-
-    def add_example_row(self, param, values):
-        """Add example row.
-
-        :param param: `str` parameter name
-        :param values: `list` of `string` parameter values
-        """
-        if param in self.example_params:
-            raise exceptions.ScenarioExamplesNotValidError(
-                """Scenario "{0}" in the feature "{1}" has not valid examples. """
-                """Example rows should contain unique parameters. {2} appeared more than once.""".format(
-                    self.name, self.feature.filename, param,
-                )
-            )
-        self.example_params.append(param)
-        self.vertical_examples.append(values)
-
-    def get_params(self):
-        """Get scenario pytest parametrization table."""
-        param_count = len(self.example_params)
-        if self.vertical_examples and not self.examples:
-            for value_index in range(len(self.vertical_examples[0])):
-                example = []
-                for param_index in range(param_count):
-                    example.append(self.vertical_examples[param_index][value_index])
-                self.examples.append(example)
-
-        if self.examples:
-            params = []
-            for example in self.examples:
-                for index, param in enumerate(self.example_params):
-                    if self.example_converters and param in self.example_converters:
-                        example[index] = self.example_converters[param](example[index])
-                params.append(example)
-            return [self.example_params, params]
-        else:
-            return []
+    def get_params(self, builtin=False):
+        """Get converted example params."""
+        for examples in [self.feature.examples, self.examples]:
+            yield examples.get_params(self.example_converters, builtin=builtin)
 
     def validate(self):
         """Validate the scenario.
@@ -405,15 +478,17 @@ class Scenario(object):
         :raises ScenarioValidationError: when scenario is not valid
         """
         params = self.params
-        if params and self.example_params and params != set(self.example_params):
+        example_params = self.get_example_params()
+        if params and example_params and params != example_params:
             raise exceptions.ScenarioExamplesNotValidError(
                 """Scenario "{0}" in the feature "{1}" has not valid examples. """
                 """Set of step parameters {2} should match set of example values {3}.""".format(
-                    self.name, self.feature.filename, sorted(params), sorted(self.example_params),
+                    self.name, self.feature.filename, sorted(params), sorted(example_params),
                 )
             )
 
 
+@six.python_2_unicode_compatible
 class Step(object):
 
     """Step."""
@@ -457,6 +532,10 @@ class Step(object):
         """Set step name."""
         self._name = value
 
+    def __str__(self):
+        """Full step name including the type."""
+        return '{type} "{name}"'.format(type=self.type.capitalize(), name=self.name)
+
     @property
     def params(self):
         """Get step params."""
