@@ -2,24 +2,18 @@
 
 import itertools
 import os.path
-import re
 
 from mako.lookup import TemplateLookup
-import glob2
 import py
 
-from .feature import Feature
 from .scenario import (
     find_argumented_step_fixture_name,
-    force_encode
+    make_python_name,
 )
+from .steps import get_step_fixture_name
+from .feature import get_features
 from .types import STEP_TYPES
 
-PYTHON_REPLACE_REGEX = re.compile("\W")
-
-ALPHA_REGEX = re.compile("^\d+_*")
-
-tw = py.io.TerminalWriter()
 
 template_lookup = TemplateLookup(directories=[os.path.join(os.path.dirname(__file__), "templates")])
 
@@ -45,18 +39,12 @@ def add_options(parser):
     )
 
 
-def pytest_cmdline_main(config):
+def cmdline_main(config):
     """Check config option to show missing code."""
     if config.option.generate_missing:
         return show_missing_code(config)
 
 
-def make_python_name(string):
-    """Make python attribute name out of a given string."""
-    string = re.sub(PYTHON_REPLACE_REGEX, "", string.replace(" ", "_"))
-    return re.sub(ALPHA_REGEX, "", string).lower()
-
-
 def generate_code(features, scenarios, steps):
     """Generate test code for the given filenames."""
     grouped_steps = group_steps(steps)
@@ -73,6 +61,7 @@ def show_missing_code(config):
 
 def print_missing_code(scenarios, steps):
     """Print missing code with TerminalWriter."""
+    tw = py.io.TerminalWriter()
     scenario = step = None
 
     for scenario in scenarios:
@@ -90,14 +79,14 @@ def print_missing_code(scenarios, steps):
         tw.line()
         if step.scenario is not None:
             tw.line(
-                """Step "{step.name}" is not defined in the scenario "{step.scenario.name}" in the feature"""
+                """Step {step} is not defined in the scenario "{step.scenario.name}" in the feature"""
                 """ "{step.scenario.feature.name}" in the file"""
                 """ {step.scenario.feature.filename}:{step.line_number}""".format(step=step),
                 red=True,
             )
         elif step.background is not None:
             tw.line(
-                """Step "{step.name}" is not defined in the background of the feature"""
+                """Step {step} is not defined in the background of the feature"""
                 """ "{step.background.feature.name}" in the file"""
                 """ {step.background.feature.filename}:{step.line_number}""".format(step=step),
                 red=True,
@@ -117,7 +106,7 @@ def print_missing_code(scenarios, steps):
     tw.write(code)
 
 
-def _find_step_fixturedef(fixturemanager, item, name, encoding="utf-8"):
+def _find_step_fixturedef(fixturemanager, item, name, type_, encoding="utf-8"):
     """Find step fixturedef.
 
     :param request: PyTest Item object.
@@ -125,38 +114,15 @@ def _find_step_fixturedef(fixturemanager, item, name, encoding="utf-8"):
 
     :return: Step function.
     """
-    fixturedefs = fixturemanager.getfixturedefs(force_encode(name, encoding), item.nodeid)
+    fixturedefs = fixturemanager.getfixturedefs(get_step_fixture_name(name, type_, encoding), item.nodeid)
     if not fixturedefs:
-        name = find_argumented_step_fixture_name(name, fixturemanager)
+        name = find_argumented_step_fixture_name(name, type_, fixturemanager)
         if name:
             return _find_step_fixturedef(fixturemanager, item, name, encoding)
     else:
         return fixturedefs
 
 
-def get_features(paths):
-    """Get features for given paths.
-
-    :param list paths: `list` of paths (file or dirs)
-
-    :return: `list` of `Feature` objects.
-    """
-    seen_names = set()
-    features = []
-    for path in paths:
-        if path in seen_names:
-            continue
-        seen_names.add(path)
-        if os.path.isdir(path):
-            features.extend(get_features(glob2.iglob(os.path.join(path, "**", "*.feature"))))
-        else:
-            base, name = os.path.split(path)
-            feature = Feature.get_feature(base, name)
-            features.append(feature)
-    features.sort(key=lambda feature: feature.name or feature.filename)
-    return features
-
-
 def parse_feature_files(paths):
     """Parse feature files of given paths.
 
@@ -194,6 +160,7 @@ def group_steps(steps):
 
 def _show_missing_code_main(config, session):
     """Preparing fixture duplicates for output."""
+    tw = py.io.TerminalWriter()
     session.perform_collect()
 
     fm = session._fixturemanager
@@ -211,7 +178,7 @@ def _show_missing_code_main(config, session):
             if scenario in scenarios:
                 scenarios.remove(scenario)
             for step in scenario.steps:
-                fixturedefs = _find_step_fixturedef(fm, item, step.name)
+                fixturedefs = _find_step_fixturedef(fm, item, step.name, step.type)
                 if fixturedefs:
                     try:
                         steps.remove(step)
