@@ -13,9 +13,13 @@ test_publish_article = scenario(
 import collections
 import inspect
 import os
+import re
 
 import pytest
-from _pytest import python
+try:
+    from _pytest import fixtures as pytest_fixtures
+except ImportError:
+    from _pytest import python as pytest_fixtures
 import six
 
 from . import exceptions
@@ -24,60 +28,33 @@ from .feature import (
     Feature,
     force_encode,
     force_unicode,
+    get_features,
 )
 from .steps import (
     execute,
     get_caller_function,
     get_caller_module,
+    get_step_fixture_name,
+    inject_fixture,
     recreate_function,
 )
 from .types import GIVEN
+from .utils import get_args, get_fixture_value
 
-
-if six.PY3:
+if six.PY3:  # pragma: no cover
     import runpy
-    execfile = runpy.run_path
+
+    def execfile(filename, init_globals):
+        """Execute given file as a python script in given globals environment."""
+        result = runpy.run_path(filename, init_globals=init_globals)
+        init_globals.update(result)
 
 
-def _inject_fixture(request, arg, value):
-    """Inject fixture into pytest fixture request.
-
-    :param request: pytest fixture request
-    :param arg: argument name
-    :param value: argument value
-    """
-    fd = python.FixtureDef(
-        request._fixturemanager,
-        None,
-        arg,
-        lambda: value, None, None,
-        False,
-    )
-    fd.cached_result = (value, 0, None)
-
-    old_fd = getattr(request, "_fixturedefs", {}).get(arg)
-    old_value = request._funcargs.get(arg)
-    add_fixturename = arg not in request.fixturenames
-
-    def fin():
-        request._fixturemanager._arg2fixturedefs[arg].remove(fd)
-        getattr(request, "_fixturedefs", {})[arg] = old_fd
-        request._funcargs[arg] = old_value
-        if add_fixturename:
-            request.fixturenames.remove(arg)
-
-    request.addfinalizer(fin)
-
-    # inject fixture definition
-    request._fixturemanager._arg2fixturedefs.setdefault(arg, []).insert(0, fd)
-    # inject fixture value in request cache
-    getattr(request, "_fixturedefs", {})[arg] = fd
-    request._funcargs[arg] = value
-    if add_fixturename:
-        request.fixturenames.append(arg)
+PYTHON_REPLACE_REGEX = re.compile("\W")
+ALPHA_REGEX = re.compile("^\d+_*")
 
 
-def find_argumented_step_fixture_name(name, fixturemanager, request=None):
+def find_argumented_step_fixture_name(name, type_, fixturemanager, request=None):
     """Find argumented step fixture name."""
     # happens to be that _arg2fixturedefs is changed during the iteration so we use a copy
     for fixturename, fixturedefs in list(fixturemanager._arg2fixturedefs.items()):
@@ -90,12 +67,12 @@ def find_argumented_step_fixture_name(name, fixturemanager, request=None):
                     if arg in converters:
                         value = converters[arg](value)
                     if request:
-                        _inject_fixture(request, arg, value)
-                parser_name = force_encode(parser.name)
+                        inject_fixture(request, arg, value)
+                parser_name = get_step_fixture_name(parser.name, type_)
                 if request:
                     try:
-                        request.getfuncargvalue(parser_name)
-                    except python.FixtureLookupError:
+                        get_fixture_value(request, parser_name)
+                    except pytest_fixtures.FixtureLookupError:
                         continue
                 return parser_name
 
@@ -112,16 +89,16 @@ def _find_step_function(request, step, scenario, encoding):
     """
     name = step.name
     try:
-        return request.getfuncargvalue(force_encode(name, encoding))
-    except python.FixtureLookupError:
+        return get_fixture_value(request, get_step_fixture_name(name, step.type, encoding))
+    except pytest_fixtures.FixtureLookupError:
         try:
-            name = find_argumented_step_fixture_name(name, request._fixturemanager, request)
+            name = find_argumented_step_fixture_name(name, step.type, request._fixturemanager, request)
             if name:
-                return request.getfuncargvalue(name)
+                return get_fixture_value(request, name)
             raise
-        except python.FixtureLookupError:
+        except pytest_fixtures.FixtureLookupError:
             raise exceptions.StepDefinitionNotFoundError(
-                u"""Step definition is not found: "{step.name}"."""
+                u"""Step definition is not found: {step}."""
                 """ Line {step.line_number} in scenario "{scenario.name}" in the feature "{feature.filename}""".format(
                     step=step,
                     scenario=scenario,
@@ -130,7 +107,7 @@ def _find_step_function(request, step, scenario, encoding):
             )
 
 
-def _execute_step_function(request, scenario, step, step_func, example=None):
+def _execute_step_function(request, scenario, step, step_func):
     """Execute step function.
 
     :param request: PyTest request.
@@ -139,51 +116,38 @@ def _execute_step_function(request, scenario, step, step_func, example=None):
     :param function step_func: Step function.
     :param example: Example table.
     """
-    request.config.hook.pytest_bdd_before_step(
-        request=request,
-        feature=scenario.feature,
-        scenario=scenario,
-        step=step,
-        step_func=step_func,
-    )
-    kwargs = {}
-    if example:
-        for key in step.params:
-            value = example[key]
-            if step_func.converters and key in step_func.converters:
-                value = step_func.converters[key](value)
-            _inject_fixture(request, key, value)
-
     kw = dict(
         request=request,
         feature=scenario.feature,
         scenario=scenario,
         step=step,
         step_func=step_func,
-        step_func_args=kwargs,
     )
 
+    request.config.hook.pytest_bdd_before_step(**kw)
+
+    kw["step_func_args"] = {}
     try:
         # Get the step argument values.
-        kwargs = dict((arg, request.getfuncargvalue(arg)) for arg in inspect.getargspec(step_func).args)
+        kwargs = dict((arg, get_fixture_value(request, arg)) for arg in get_args(step_func))
+        kw["step_func_args"] = kwargs
 
+        request.config.hook.pytest_bdd_before_step_call(**kw)
         # Execute the step.
         step_func(**kwargs)
-        kw["step_func_args"] = kwargs
         request.config.hook.pytest_bdd_after_step(**kw)
     except Exception as exception:
         request.config.hook.pytest_bdd_step_error(exception=exception, **kw)
         raise
 
 
-def _execute_scenario(feature, scenario, request, encoding, example=None):
+def _execute_scenario(feature, scenario, request, encoding):
     """Execute the scenario.
 
     :param feature: Feature.
     :param scenario: Scenario.
     :param request: request.
     :param encoding: Encoding.
-    :param example: Example.
     """
     request.config.hook.pytest_bdd_before_scenario(
         request=request,
@@ -208,12 +172,6 @@ def _execute_scenario(feature, scenario, request, encoding, example=None):
                 raise
 
             try:
-                # Check the step types are called in the correct order
-                if step_func.step_type != step.type:
-                    raise exceptions.StepTypeError(
-                        'Wrong step type "{0}" while "{1}" is expected.'.format(step_func.step_type, step.type)
-                    )
-
                 # Check if the fixture that implements given step has not been yet used by another given step
                 if step.type == GIVEN:
                     if step_func.fixture in givens:
@@ -234,7 +192,7 @@ def _execute_scenario(feature, scenario, request, encoding, example=None):
                 )
                 raise
 
-            _execute_step_function(request, scenario, step, step_func, example=example)
+            _execute_step_function(request, scenario, step, step_func)
     finally:
         request.config.hook.pytest_bdd_after_scenario(
             request=request,
@@ -250,7 +208,7 @@ def get_fixture(caller_module, fixture, path=None):
     """Get first conftest module from given one."""
     def call_fixture(function):
         args = []
-        if "request" in inspect.getargspec(function).args:
+        if "request" in get_args(function):
             args = [FakeRequest(module=caller_module)]
         return function(*args)
 
@@ -267,7 +225,7 @@ def get_fixture(caller_module, fixture, path=None):
             if fixture in globs:
                 return call_fixture(globs[fixture])
     else:
-        return call_fixture(fixtures.pytestbdd_feature_base_dir)
+        return call_fixture(getattr(fixtures, fixture))
     return get_fixture(caller_module, fixture, path=os.path.dirname(path))
 
 
@@ -279,16 +237,16 @@ def _get_scenario_decorator(feature, feature_name, scenario, scenario_name, call
     scenario_name = force_encode(scenario_name, encoding)
 
     def decorator(_pytestbdd_function):
-        if isinstance(_pytestbdd_function, python.FixtureRequest):
+        if isinstance(_pytestbdd_function, pytest_fixtures.FixtureRequest):
             raise exceptions.ScenarioIsDecoratorOnly(
                 "scenario function can only be used as a decorator. Refer to the documentation.",
             )
 
         g.update(locals())
 
-        args = inspect.getargspec(_pytestbdd_function).args
+        args = get_args(_pytestbdd_function)
         function_args = list(args)
-        for arg in scenario.example_params:
+        for arg in scenario.get_example_params():
             if arg not in function_args:
                 function_args.append(arg)
         if "request" not in function_args:
@@ -309,13 +267,12 @@ def _get_scenario_decorator(feature, feature_name, scenario, scenario_name, call
             firstlineno=caller_function.f_lineno,
         )
 
-        params = scenario.get_params()
-
-        if params:
-            _scenario = pytest.mark.parametrize(*params)(_scenario)
+        for param_set in scenario.get_params():
+            if param_set:
+                _scenario = pytest.mark.parametrize(*param_set)(_scenario)
 
         for tag in scenario.tags.union(feature.tags):
-            _scenario = getattr(pytest.mark, tag)(_scenario)
+            pytest.config.hook.pytest_bdd_apply_tag(tag=tag, function=_scenario)
 
         _scenario.__doc__ = "{feature_name}: {scenario_name}".format(
             feature_name=feature_name, scenario_name=scenario_name)
@@ -328,14 +285,22 @@ def _get_scenario_decorator(feature, feature_name, scenario, scenario_name, call
 
 def scenario(feature_name, scenario_name, encoding="utf-8", example_converters=None,
              caller_module=None, caller_function=None):
-    """Scenario."""
+    """Scenario decorator.
+
+    :param str feature_name: Feature file name. Absolute or relative to the configured feature base path.
+    :param str scenario_name: Scenario name.
+    :param str encoding: Feature file encoding.
+    :param dict example_converters: optional `dict` of example converter function, where key is the name of the
+        example parameter, and value is the converter function.
+    """
     scenario_name = force_unicode(scenario_name, encoding)
     caller_module = caller_module or get_caller_module()
     caller_function = caller_function or get_caller_function()
 
     # Get the feature
     base_path = get_fixture(caller_module, "pytestbdd_feature_base_dir")
-    feature = Feature.get_feature(base_path, feature_name, encoding=encoding)
+    strict_gherkin = get_fixture(caller_module, "pytestbdd_strict_gherkin")
+    feature = Feature.get_feature(base_path, feature_name, encoding=encoding, strict_gherkin=strict_gherkin)
 
     # Get the sc_enario
     try:
@@ -363,3 +328,64 @@ def scenario(feature_name, scenario_name, encoding="utf-8", example_converters=N
         caller_function,
         encoding,
     )
+
+
+def make_python_name(string):
+    """Make python attribute name out of a given string."""
+    string = re.sub(PYTHON_REPLACE_REGEX, "", string.replace(" ", "_"))
+    return re.sub(ALPHA_REGEX, "", string).lower()
+
+
+def get_python_name_generator(name):
+    """Generate a sequence of suitable python names out of given arbitrary string name."""
+    python_name = make_python_name(name)
+    suffix = ''
+    index = 0
+
+    def get_name():
+        return 'test_{0}{1}'.format(python_name, suffix)
+    while True:
+        yield get_name()
+        index += 1
+        suffix = '_{0}'.format(index)
+
+
+def scenarios(*feature_paths, **kwargs):
+    """Parse features from the paths and put all found scenarios in the caller module.
+
+    :param *feature_paths: feature file paths to use for scenarios
+    """
+    frame = inspect.stack()[1]
+    module = inspect.getmodule(frame[0])
+    base_path = get_fixture(module, "pytestbdd_feature_base_dir")
+    strict_gherkin = get_fixture(module, "pytestbdd_strict_gherkin")
+    abs_feature_paths = []
+    for path in feature_paths:
+        if not os.path.isabs(path):
+            path = os.path.abspath(os.path.join(base_path, path))
+        abs_feature_paths.append(path)
+    found = False
+
+    module_scenarios = frozenset(
+        (attr.__scenario__.feature.filename, attr.__scenario__.name)
+        for name, attr in module.__dict__.items() if hasattr(attr, '__scenario__'))
+
+    index = 10
+    for feature in get_features(abs_feature_paths, strict_gherkin=strict_gherkin):
+        for scenario_name, scenario_object in feature.scenarios.items():
+            # skip already bound scenarios
+            if (scenario_object.feature.filename, scenario_name) not in module_scenarios:
+                @scenario(feature.filename, scenario_name, **kwargs)
+                def _scenario():
+                    pass  # pragma: no cover
+                for test_name in get_python_name_generator(scenario_name):
+                    if test_name not in module.__dict__:
+                        # found an unique test name
+                        # recreate function to set line number
+                        _scenario = recreate_function(_scenario, module=module, firstlineno=index * 4)
+                        index += 1
+                        module.__dict__[test_name] = _scenario
+                        break
+            found = True
+    if not found:
+        raise exceptions.NoScenariosFound(abs_feature_paths)
