@@ -37,6 +37,10 @@ import inspect
 import sys
 
 import pytest
+try:
+    from _pytest import fixtures as pytest_fixtures
+except ImportError:
+    from _pytest import python as pytest_fixtures
 import six
 
 from .feature import parse_line, force_encode
@@ -45,34 +49,52 @@ from .exceptions import (
     StepError,
 )
 from .parsers import get_parser
+from .utils import get_args, get_fixture_value, get_fixture_value_raw, set_fixture_value, get_request_fixture_defs, \
+    get_request_fixture_names
 
 
-def given(name, fixture=None, converters=None):
+def get_step_fixture_name(name, type_, encoding=None):
+    """Get step fixture name.
+
+    :param name: unicode string
+    :param type: step type
+    :param encoding: encoding
+    :return: step fixture name
+    :rtype: string
+    """
+    return "pytestbdd_{type}_{name}".format(
+        type=type_, name=force_encode(name, **(dict(encoding=encoding) if encoding else {})))
+
+
+def given(name, fixture=None, converters=None, scope='function', target_fixture=None):
     """Given step decorator.
 
     :param name: Given step name.
     :param fixture: Optional name of the fixture to reuse.
     :param converters: Optional `dict` of the argument or parameter converters in form
                        {<param_name>: <converter function>}.
-    :param parser: name of the step parser to use
-    :param parser_args: optional `dict` of arguments to pass to step parser
+    :scope: Optional fixture scope
+    :param target_fixture: Target fixture name to replace by steps definition function
     :raises: StepError in case of wrong configuration.
     :note: Can't be used as a decorator when the fixture is specified.
     """
     if fixture is not None:
         module = get_caller_module()
-        step_func = lambda request: request.getfuncargvalue(fixture)
+
+        def step_func(request):
+            return get_fixture_value(request, fixture)
+
         step_func.step_type = GIVEN
         step_func.converters = converters
         step_func.__name__ = name
         step_func.fixture = fixture
-        func = pytest.fixture(lambda: step_func)
+        func = pytest.fixture(scope=scope)(lambda: step_func)
         func.__doc__ = 'Alias for the "{0}" fixture.'.format(fixture)
         _, name = parse_line(name)
-        contribute_to_module(module, name, func)
+        contribute_to_module(module, get_step_fixture_name(name, GIVEN), func)
         return _not_a_fixture_decorator
 
-    return _step_decorator(GIVEN, name, converters=converters)
+    return _step_decorator(GIVEN, name, converters=converters, scope=scope, target_fixture=target_fixture)
 
 
 def when(name, converters=None):
@@ -113,13 +135,14 @@ def _not_a_fixture_decorator(func):
     raise StepError('Cannot be used as a decorator when the fixture is specified')
 
 
-def _step_decorator(step_type, step_name, converters=None):
+def _step_decorator(step_type, step_name, converters=None, scope='function', target_fixture=None):
     """Step decorator for the type and the name.
 
     :param str step_type: Step type (GIVEN, WHEN or THEN).
     :param str step_name: Step name as in the feature file.
-    :param str parser: name of the step parser to use
-    :param dict parser_args: optional `dict` of arguments to pass to step parser
+    :param dict converters: Optional step arguments converters mapping
+    :param str scope: Optional step definition fixture scope
+    :param target_fixture: Optional fixture name to replace by step definition
 
     :return: Decorator function for the step.
 
@@ -136,20 +159,27 @@ def _step_decorator(step_type, step_name, converters=None):
         if step_type == GIVEN:
             if not hasattr(func, "_pytestfixturefunction"):
                 # Avoid multiple wrapping of a fixture
-                func = pytest.fixture(func)
-            step_func = lambda request: request.getfuncargvalue(func.__name__)
+                func = pytest.fixture(scope=scope)(func)
+
+            def step_func(request):
+                result = get_fixture_value(request, func.__name__)
+                if target_fixture:
+                    inject_fixture(request, target_fixture, result)
+                return result
+
             step_func.__doc__ = func.__doc__
             step_func.fixture = func.__name__
 
         step_func.__name__ = force_encode(parsed_step_name)
 
-        @pytest.fixture
+        @pytest.fixture(scope=scope)
         def lazy_step_func():
             return step_func
 
         step_func.step_type = step_type
 
-        lazy_step_func = contribute_to_module(get_caller_module(), parsed_step_name, lazy_step_func)
+        lazy_step_func = contribute_to_module(
+            get_caller_module(), get_step_fixture_name(parsed_step_name, step_type), lazy_step_func)
 
         lazy_step_func.step_type = step_type
 
@@ -190,7 +220,7 @@ def recreate_function(func, module=None, name=None, add_args=[], firstlineno=Non
     if six.PY3:
         argnames.insert(1, "co_kwonlyargcount")
 
-    for arg in inspect.getargspec(func).args:
+    for arg in get_args(func):
         if arg in add_args:
             add_args.remove(arg)
 
@@ -237,7 +267,7 @@ def get_caller_module(depth=2):
     frame = sys._getframe(depth)
     module = inspect.getmodule(frame)
     if module is None:
-        raise Exception("empty module")
+        return get_caller_module(depth=depth)
     return module
 
 
@@ -247,4 +277,49 @@ def get_caller_function(depth=2):
 
 
 def execute(code, g):
+    """Execute given code in given globals environment."""
     exec(code, g)
+
+
+def inject_fixture(request, arg, value):
+    """Inject fixture into pytest fixture request.
+
+    :param request: pytest fixture request
+    :param arg: argument name
+    :param value: argument value
+    """
+    fd_kwargs = {
+        'fixturemanager': request._fixturemanager,
+        'baseid': None,
+        'argname': arg,
+        'func': lambda: value,
+        'scope': "function",
+        'params': None,
+    }
+
+    if 'yieldctx' in get_args(pytest_fixtures.FixtureDef.__init__):
+        fd_kwargs['yieldctx'] = False
+
+    fd = pytest_fixtures.FixtureDef(**fd_kwargs)
+    fd.cached_result = (value, 0, None)
+
+    old_fd = get_request_fixture_defs(request).get(arg)
+    old_value = get_fixture_value_raw(request, arg)
+    add_fixturename = arg not in request.fixturenames
+
+    def fin():
+        request._fixturemanager._arg2fixturedefs[arg].remove(fd)
+        get_request_fixture_defs(request)[arg] = old_fd
+        set_fixture_value(request, arg, old_value)
+        if add_fixturename:
+            get_request_fixture_names(request).remove(arg)
+
+    request.addfinalizer(fin)
+
+    # inject fixture definition
+    request._fixturemanager._arg2fixturedefs.setdefault(arg, []).insert(0, fd)
+    # inject fixture value in request cache
+    get_request_fixture_defs(request)[arg] = fd
+    set_fixture_value(request, arg, value)
+    if add_fixturename:
+        get_request_fixture_names(request).append(arg)
